@@ -5,13 +5,15 @@
  * Note to U.S. Government Users Restricted Rights:
  * Use, duplication or disclosure restricted by GSA ADP Schedule
  * Contract with IBM Corp.
- * Copyright (c) 2020 Red Hat, Inc.
  ****************************************************************************** */
+// Copyright (c) 2020 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 /* eslint no-param-reassign: "error" */
 import _ from 'lodash';
 import { responseHasError } from '../lib/utils';
 import logger from '../lib/logger';
 import GenericModel from './generic';
+import { getStatus } from './cluster';
 
 export const ALL_SUBSCRIPTIONS = '__ALL__/SUBSCRIPTIONS__';
 const EVERYTHING_CHANNEL = '__ALL__/__ALL__//__ALL__/__ALL__';
@@ -264,6 +266,19 @@ export const buildDeployablesMap = (subscriptions, modelSubscriptions) => {
   };
 };
 
+export const evaluateSingleAnd = (operand1, operand2) => operand1 && operand2;
+
+export const evaluateDoubleAnd = (operand1, operand2, operand3) => operand1 && operand2 && operand3;
+
+export const evaluateSingleOr = (operand1, operand2) => operand1 || operand2;
+
+export const evaluateTernaryExpression = (condition, returnVal1, returnVal2) => {
+  if (condition) {
+    return returnVal1;
+  }
+  return returnVal2;
+};
+
 export default class ApplicationModel extends GenericModel {
   // ///////////// CREATE APPLICATION ////////////////
   // ///////////// CREATE APPLICATION ////////////////
@@ -384,7 +399,7 @@ export default class ApplicationModel extends GenericModel {
     const namespaceResponses = await Promise.all(namespaces.map((ns) => {
       let namespaceExists = false;
       if (existingNamespaces && existingNamespaces.items) {
-        namespaceExists = existingNamespaces.items.find((existingNamespace) => existingNamespace.metadata.name === ns);
+        namespaceExists = existingNamespaces.items.find((existingNamespace) => _.get(existingNamespace, 'metadata.name', '') === ns);
       }
       return !namespaceExists ? this.createNamespace(ns) : undefined;
     }));
@@ -456,6 +471,8 @@ export default class ApplicationModel extends GenericModel {
 
     // if everything else created/updated
     if (errors.length === 0) {
+      // create ansible secrets if any are used and not yet available in the app ns
+      await this.createAnsibleSecrets(application);
       // if update
       if (isEdit) {
         // delete any removed resources except placement rules
@@ -534,15 +551,49 @@ export default class ApplicationModel extends GenericModel {
   // ///////////// GET APPLICATION ////////////////
 
   // for topology and editor pages
-  async getApplication(name, namespace, selectedChannel, includeChannels) {
+  async getApplication(name, namespace, selectedChannel, includeChannels, cluster, apiversion) {
     // get application
     let model = null;
-    let apps;
+    let apps = [];
+    const apiVersion = apiversion || 'app.k8s.io/v1beta1'; // defaults to ACM app
     try {
-      apps = await this.kubeConnector.getResources(
-        (ns) => `/apis/app.k8s.io/v1beta1/namespaces/${ns}/applications/${name}`,
-        { namespaces: [namespace] },
-      );
+      if (cluster) {
+        // find Argo app on remote cluster
+        const args = {
+          apiVersion,
+          cluster,
+          kind: 'application',
+          name,
+          namespace,
+        };
+        const result = await this.getResource(args);
+        if (!result) {
+          const managedCluster = await this.kubeConnector.get(
+            `/apis/cluster.open-cluster-management.io/v1/managedclusters/${cluster}`,
+          );
+
+          const clusterStatus = getStatus(managedCluster);
+          let msgcode;
+
+          if (clusterStatus === 'offline') {
+            msgcode = 'load.app.cluster.offline';
+          }
+          model = {
+            error: {
+              msgcode,
+              args: [cluster],
+            },
+          };
+          return model;
+        }
+        apps.push(result);
+      } else {
+        // get app from local hub
+        apps = await this.kubeConnector.getResources(
+          (ns) => `/apis/${apiVersion}/namespaces/${ns}/applications/${name}`,
+          { namespaces: [namespace] },
+        );
+      }
     } catch (err) {
       logger.error(err);
       throw err;
@@ -556,17 +607,19 @@ export default class ApplicationModel extends GenericModel {
         name, namespace, app, metadata: app.metadata,
       };
 
+      if (_.get(app, 'apiVersion', '').indexOf('argoproj.io') > -1) {
+        return model;
+      }
+
       // get subscriptions to channels (pipelines)
-      let subscriptionNames = _.get(app, 'metadata.annotations["apps.open-cluster-management.io/subscriptions"]')
-        || _.get(app, 'metadata.annotations["app.ibm.com/subscriptions"]');
-      let deployableNames = _.get(app, DEPLOYABLES)
-        || _.get(app, 'metadata.annotations["app.ibm.com/subscriptions/deployables"]');
+      let subscriptionNames = _.get(app, 'metadata.annotations["apps.open-cluster-management.io/subscriptions"]');
+      let deployableNames = _.get(app, DEPLOYABLES);
       if (subscriptionNames && subscriptionNames.length > 0) {
         subscriptionNames = subscriptionNames.split(',');
         // filter local hub subscription
         const filteredSubscriptions = [];
         subscriptionNames.forEach((subscriptionName) => {
-          if (!(_.endsWith(subscriptionName, '-local') && _.indexOf(subscriptionNames, _.trimEnd(subscriptionName, '-local')) !== -1)) {
+          if (!(evaluateSingleAnd(_.endsWith(subscriptionName, '-local'), _.indexOf(subscriptionNames, _.trimEnd(subscriptionName, '-local')) !== -1))) {
             filteredSubscriptions.push(subscriptionName);
           }
         });
@@ -591,10 +644,10 @@ export default class ApplicationModel extends GenericModel {
         );
 
         // get all requested subscriptions
-        const selectedSubscription = selectedChannel === ALL_SUBSCRIPTIONS ? allSubscriptions : subscr;
+        const selectedSubscription = evaluateTernaryExpression(selectedChannel === ALL_SUBSCRIPTIONS, allSubscriptions, subscr);
         const {
           deployableMap, channelsMap, rulesMap, preHooksMap, postHooksMap,
-        } = buildDeployablesMap(selectedSubscription || subscriptions, model.subscriptions);
+        } = buildDeployablesMap(evaluateSingleOr(selectedSubscription, subscriptions), model.subscriptions);
         // now fetch them
         await this.getAppDeployables(deployableMap, namespace, selectedSubscription, subscriptions);
         await this.getAppHooks(preHooksMap, true);
@@ -614,6 +667,20 @@ export default class ApplicationModel extends GenericModel {
     return model;
   }
 
+  async getApplications() {
+    const apps = _.flatten(await Promise.all([
+      this.kubeConnector.getResources((ns) => `/apis/app.k8s.io/v1beta1/namespaces/${ns}/applications`),
+      this.kubeConnector.getResources((ns) => `/apis/argoproj.io/v1alpha1/namespaces/${ns}/applications`),
+    ]).catch((err) => {
+      logger.error(err);
+      throw err;
+    }));
+    return _.sortBy(apps.map((app) => ({
+      metadata: app.metadata,
+      raw: app,
+    })), ['metadata.name', 'metadata.namespace']);
+  }
+
   async getAppDeployables(deployableMap) {
     const requests = Object.entries(deployableMap).map(async ([namespace, values]) => {
       // get all deployables in this namespace
@@ -811,15 +878,155 @@ export default class ApplicationModel extends GenericModel {
     return _.filter(namespaces, (ns) => !_.get(ns, 'metadata.name', '').startsWith('openshift') && !_.get(ns, 'metadata.name', '').startsWith('open-cluster-management'));
   }
 
-  async getSecrets(namespace) {
-    const secrets = await this.kubeConnector.getResources((ns) => `/api/v1/namespaces/${ns}/secrets`, { namespaces: [namespace] }).catch((err) => {
+  async getManagedCluster(clusterName) {
+    const managedCluster = await this.kubeConnector.get(
+      `/apis/cluster.open-cluster-management.io/v1/managedclusters/${clusterName}`,
+    ).catch((err) => {
       logger.error(err);
       throw err;
     });
-    return secrets.filter((secret) => secret.metadata && _.get(secret, 'data.host') && _.get(secret, 'data.token'))
-      .map((secret) => ({
-        name: _.get(secret, 'metadata.name', 'unknown'),
-        namespace: _.get(secret, 'metadata.name', 'unknown'),
-      }));
+    let successImportStatus = false;
+    const managedClusterCondition = _.get(managedCluster, 'status.conditions', {});
+    if (!_.isEmpty(managedClusterCondition)) {
+      // check cluster import condition
+      const managedClusterAvailable = _.find(managedClusterCondition, (condition) => condition.type === 'ManagedClusterConditionAvailable');
+      if (managedClusterAvailable && _.has(managedClusterAvailable, 'status')) {
+        successImportStatus = _.get(managedClusterAvailable, 'status') === 'True' ? true : successImportStatus;
+      }
+    }
+    return successImportStatus;
+  }
+
+  async getSecrets(labelObject) {
+    const { label, value } = labelObject;
+    const secrets = await this.kubeConnector.get(`/api/v1/secrets/?labelSelector=${encodeURIComponent(label)}`).catch((err) => {
+      logger.error(err);
+      throw err;
+    });
+    const ansibleSecrets = _.filter(_.get(secrets, 'items', []), (secret) => _.get(secret, 'metadata.labels', {})[label] === value);
+    return ansibleSecrets.map((secret) => ({
+      ansibleSecretName: _.get(secret, 'metadata.name', 'unknown'),
+      ansibleSecretNamespace: _.get(secret, 'metadata.namespace', 'unknown'),
+    }));
+  }
+
+  async createAnsibleSecrets(application) {
+    // get all subscriptions using an ansible provider
+    const nsPath = 'metadata.namespace';
+    const namePath = 'metadata.name';
+    const subsUsingAnsible = _.filter(application, (resource) => _.get(resource, 'kind', '') === 'Subscription'
+          && _.get(resource, 'spec.hooksecretref.name'));
+    const ansibleSecretNames = _.uniqBy(_.map(subsUsingAnsible, 'spec.hooksecretref.name'));
+    const applicationResource = _.find(application, (res) => _.get(res, 'kind', '') === 'Application');
+    const namespace = _.get(applicationResource, nsPath, '');
+    const ansibleSelector = {
+      label: 'cluster.open-cluster-management.io/type',
+      value: 'ans',
+    };
+    if (!ansibleSecretNames || ansibleSecretNames.length === 0) {
+      // no secrets to create
+      return;
+    }
+    ansibleSecretNames.forEach(async (name) => {
+      // check if a secret with this name already exists in the app ns
+      try {
+        const secrets = await this.kubeConnector.get(`/api/v1/secrets/?labelSelector=${encodeURIComponent(ansibleSelector.label)}`).catch((err) => {
+          logger.error(err);
+          throw err;
+        });
+        const secretInAppNS = _.find(_.get(secrets, 'items', []), (obj) => _.get(obj, namePath, '') === name && _.get(obj, nsPath, '') === namespace);
+        if (!secretInAppNS || secretInAppNS.length === 0) {
+          const ansibleSecrets = _.filter(_.get(secrets, 'items', []), (secret) => _.get(secret, namePath, '') === name && _.get(secret, 'metadata.labels', {})[ansibleSelector.label] === ansibleSelector.value);
+          if (ansibleSecrets.length > 0) {
+            // now create the secret crd in the app ns
+            try {
+              const apiVersion = _.get(ansibleSecrets[0], 'apiVersion', 'v1');
+              const kind = 'Secret';
+              const resource = {
+                apiVersion,
+                kind,
+                metadata: {
+                  name,
+                  namespace,
+                  labels: {
+                    'cluster.open-cluster-management.io/copiedFromNamespace': _.get(ansibleSecrets[0], 'metadata.namespace', ''),
+                    'cluster.open-cluster-management.io/copiedFromSecretName': _.get(ansibleSecrets[0], 'metadata.name', ''),
+                  },
+                },
+                data: _.pick(_.get(ansibleSecrets[0], 'data', {}), ['host', 'token']),
+              };
+              const requestPath = await this.getResourceEndPoint(resource);
+              await this.kubeConnector.post(requestPath, resource);
+            } catch (err) {
+              logger.error(err);
+            }
+          }
+        }
+      } catch (err) {
+        logger.error(err);
+      }
+    });
+  }
+
+  async getLocalArgoRoute(args) {
+    const routes = await this.kubeConnector.getResources((ns) => `/apis/${args.apiVersion}/namespaces/${ns}/routes`, { namespaces: [args.namespace], kind: args.kind }).catch((err) => {
+      logger.error(err);
+      throw err;
+    });
+
+    if (routes && routes.length) {
+      // route exists
+      const routeObjs = routes
+        .filter((route) => _.get(route, 'metadata.labels["app.kubernetes.io/part-of"]', '') === 'argocd'
+          && !_.get(route, 'metadata.name', '').toLowerCase().includes('grafana')
+          && !_.get(route, 'metadata.name', '').toLowerCase().includes('prometheus'));
+      if (routeObjs.length > 0) {
+        // if still more than 1, choose one with “server” in the name if possible
+        const serverRoute = routeObjs.find((route) => _.get(route, 'metadata.name', '').toLowerCase().includes('server'));
+        if (serverRoute) {
+          return serverRoute;
+        }
+        return routeObjs[0];
+      }
+      return undefined;
+    }
+    // route doesn't exist
+    return undefined;
+  }
+
+  // returns the url for the ARGO CD editor
+  async getArgoAppRouteURL(variables) {
+    const args = {
+      ...variables,
+      kind: 'route',
+    };
+    let route;
+    if (variables.cluster === 'local-cluster') {
+      route = await this.getLocalArgoRoute(args);
+    } else {
+      route = await this.getResource(args);
+    }
+
+    if (!route) {
+      return '';
+    }
+    const hostName = _.get(route, 'spec.host', 'unknown');
+    const transport = _.get(route, 'spec.tls') ? 'https' : 'http';
+    return `${transport}://${hostName}/applications`;
+  }
+
+  // returns the url for Route resource
+  async getRouteResourceURL(variables) {
+    const args = {
+      ...variables,
+      kind: 'route',
+    };
+    const route = await this.getResource(args);
+    if (!route) {
+      return '';
+    }
+    const hostName = _.get(route, 'spec.host', 'unknown');
+    const transport = _.get(route, 'spec.tls') ? 'https' : 'http';
+    return `${transport}://${hostName}`;
   }
 }
